/**
 * @file
 * Content locker consent js.
 */

(function ($, window, Drupal) {
  Drupal.behaviors.lockerConsent = {
    attach: function attach(context, settings) {
      if (typeof Drupal.Locker !== 'undefined') {
        
        function LockerConsent(holder) {
          Drupal.Locker.apply(this, arguments);
        }

        LockerConsent.prototype = Object.create(new Drupal.Locker());
        LockerConsent.prototype.constructor = LockerConsent;

        /**
         * Render consent buttons
         * @param holder
         */
        LockerConsent.prototype.render = function (holder) {
          if (holder.length) {
            this.holder = holder;
            var lock = $('<span>')
              .addClass(this.iconClass);
            var choice1 = $('<button>')
              .addClass('cl-control')
              .addClass('cl-' + this.type)
              .addClass('locker-btn')
              .attr('value', '1')
              .html('Yes');

            var choice2 = $('<button>')
              .addClass('cl-control')
              .addClass('cl-' + this.type)
              .addClass('locker-btn')
              .attr('value', '0')
              .html('No');

            var wrapRadios = $('<div>')
              .addClass('cl-wrapper')
              .addClass(this.actionClass);


            var errors = $('<div>')
              .addClass(this.errorClass)
              .addClass('element-hidden');
            lock.appendTo(holder)
            errors.appendTo(holder);
            var lockerOptionsGeneral = Drupal.getLockerOptions(settings, 'consent', 'general');
            $('<div>')
              .addClass(this.contentClass)
              .html(lockerOptionsGeneral.text.value).appendTo(holder);
            choice1.appendTo(wrapRadios);
            choice2.appendTo(wrapRadios);
            wrapRadios.appendTo(holder)
          }
          else {
            throw new Error(
              'holder should not be empty.'
            );
          }
        };

        /**
         * Check if content is locked
         *
         * @param holder
         * @returns {*}
         */
        LockerConsent.prototype.isLocked = function (holder) {
          var control = $('input[name="choice-radio"]:checked', holder);
          if (control && control.length) {
            return control.value;
          }
          return 1;
        };

        /**
         * Locker content action rejected by user.
         *
         * @param e Event
         */
        LockerConsent.prototype.rejectEvent = function (e) {
          var err = $('.' + this.errorClass, $(this.holder));
          var errmsg;
          if (this.options.general.error) {
            errmsg = this.options.general.error
          }
          else {
            errmsg = "Locker can not be unlocked."
          }
          if (err && err.length) {
            err.html(errmsg);
            err.removeClass('element-hidden')
          }
        };
      }

      var lockerPlaceholder = $('.content-locker');
      if (lockerPlaceholder && lockerPlaceholder.length) {
        lockerPlaceholder.once('content-locker-consent').each(function (i, el) {
          var holder = $(el), type = holder.data('content-locker-type'), lockerConsent;
          if (type === 'consent') {
            var baseOptions, lockerOptions;
            if (typeof Drupal.Locker !== 'undefined') {
              baseOptions = Drupal.getLockerOptions(settings, 'consent', 'base');
              lockerOptions = Drupal.getLockerOptions(settings, 'consent');
              lockerConsent = new LockerConsent(holder);

              if (lockerOptions) {
                var lockerSettings = {
                  type: type,
                  options: lockerOptions,
                  errorClass: 'locker-error',
                  holder: holder,
                  contentClass: 'locker-content',
                  iconClass: 'icon-lock',
                  actionClass: 'locker-actions'
                };

                lockerConsent.setOptions(lockerSettings);
                lockerConsent.isAjax = baseOptions.ajax;
                lockerConsent.isCookie = baseOptions.cookie;
                lockerConsent.cookieLife = baseOptions.cookie_lifetime;
                var uuid = Drupal.userId();
                lockerConsent.errorClass = lockerSettings.errorClass;
                lockerConsent.setUserId(uuid);
                lockerConsent.subscribeEvents(holder, lockerConsent);
                lockerConsent.render(holder);
              }

              var radios = $('.locker-btn', holder);
              if (radios && radios.length) {
                radios.on('click', function (e) {
                  e.preventDefault();
                  var value = $(e.target).attr('value');
                  switch (value) {
                    case '1':
                      lockerConsent.hideError(e);
                      lockerConsent.fireEvent(lockerConsent.unlockevent, holder,
                        this.value);
                      return false;
                    case '0':
                      lockerConsent.rejectEvent(e);
                      return false;
                  }
                  return false;
                });

              }

              if (lockerConsent
                && lockerConsent.isLocked(holder)) {
                lockerConsent.fireEvent(lockerConsent.lockevent, holder, lockerConsent.type)
              }
              else {
                lockerConsent.fireEvent(lockerConsent.unlockevent, holder, lockerConsent.type)
              }
            }
          }
        });
      }
    }
  };
  
})(jQuery, window, Drupal);