/**
 * @file
 * Content locker js.
 */

(function ($, window, Drupal) {

  /**
   * Insert new content into the DOM.
   */
  if (Drupal.AjaxCommands) {
    Drupal.AjaxCommands.prototype.contentLockerUpdateEntity = function (ajax, response, status) {
      response.selector = '[data-content-locker-' + response.entityType + '-id=' + response.entityId + ']';
      this.insert(ajax, response, status);
      $(document).trigger('contentLockerUpdateEntity',
        {
          ltype: response.lockerType,
          etype: response.entityType,
          nid: response.entityId
        },
        {
          error: response.error
        }
      );
    };
  }

  Drupal.behaviors.contentLocker = {
    ajaxed: [],
    attach: function attach(context, settings) {
       $(document).once('content-locker', function (){
         Drupal.behaviors.contentLocker.ajaxed = [];
       });
      //Object.keys polyfil
      if (!Object.keys) {
        Object.keys = (function() {
          'use strict';
          var hasOwnProperty = Object.prototype.hasOwnProperty,
            hasDontEnumBug = !({ toString: null }).propertyIsEnumerable('toString'),
            dontEnums = [
              'toString',
              'toLocaleString',
              'valueOf',
              'hasOwnProperty',
              'isPrototypeOf',
              'propertyIsEnumerable',
              'constructor'
            ],
            dontEnumsLength = dontEnums.length;

          return function(obj) {
            if (typeof obj !== 'object' && (typeof obj !== 'function' || obj === null)) {
              throw new TypeError('Object.keys called on non-object');
            }

            var result = [], prop, i;

            for (prop in obj) {
              if (hasOwnProperty.call(obj, prop)) {
                result.push(prop);
              }
            }

            if (hasDontEnumBug) {
              for (i = 0; i < dontEnumsLength; i++) {
                if (hasOwnProperty.call(obj, dontEnums[i])) {
                  result.push(dontEnums[i]);
                }
              }
            }
            return result;
          };
        }());
      }


      // forEach polyfill
      if (!Array.prototype.forEach) {
        Array.prototype.forEach = function(fn, scope) {
          for (var i = 0, len = this.length; i < len; ++i) {
            fn.call(scope, this[i], i, this);
          }
        }
      }
    }
  };

  

  Drupal.Locker = function (type) {
    if (type && type.length) {
      this.type = type;
    }
    this.lockevent = 'lock';
    this.unlockevent = 'unlock';
    this.lockedContentClass = '.locked-content';
    this.errorClass = 'locker-error'
  };

  Drupal.Locker.prototype.log = function () {
    if (console) {
      console.dir(this);
    }                                                         
  };

  Drupal.Locker.prototype.setOptions = function (obj) {
    var self = this;
    if (obj) {
      Object.keys(obj).map(function(objKey, i) {
        self[objKey] = obj[objKey];
      });
    }
  };

  Drupal.Locker.prototype.setUserId = function (uuid) {
    this.uuid = uuid;
  };

  Drupal.Locker.prototype.lockEventAction = function (e, obj) {
    var isAjax = (obj.isAjax) ? obj.isAjax : 0;
    var target = $(obj.lockedContentClass, obj.holder);
    if (!isAjax && target && target.length) {
      if (!Drupal.elementIsHidden(target)) {
        target.hide();
      }
    }
  };

  Drupal.Locker.prototype.unlockEventAction = function (e, obj, nid) {
    var isAjax = (obj.isAjax) ? obj.isAjax : null;
    var target = $(obj.lockedContentClass, obj.holder);
    var entity, uid =  Drupal.userId();
    if (isAjax && target && target.length) {
      var ajaxed = Drupal.behaviors.contentLocker.ajaxed;
      if (ajaxed.indexOf(obj.type) === -1) {
        ajaxed.push(obj.type);
      }
      var ajaxObject = Drupal.ajax({
        url: '',
        base: false,
        element: false,
        progress: false,
        dialogType: 'content_locker',
        dialog : {types: ajaxed}
      });
      ajaxObject.execute();
    }
    else if (!isAjax && target && target.length){
      if (Drupal.elementIsHidden(target)) {
        this.hideLocker();
        entity = Drupal.getEntityData();
        if (entity) {
          this.onUpdateNode('unlock', entity.type, entity.id)
        }
        else {
          throw new Error('Can not get entity.');
        }
        target.show();
        this.clearContent(target, obj.holder)
      }
    }
  };

  Drupal.Locker.prototype.clearContent = function (target, context) {
    $('.' + this.iconClass, context).remove();
    $('.' + this.errorClass, context).remove();
    $('.' + this.contentClass, context).remove();
    $('.' + this.actionClass, context).remove();
    target.unwrap();
  };

  Drupal.Locker.prototype.isLocked = function () {
  };

  Drupal.Locker.prototype.fireEvent = function (type, el, value) {
    if (!el) el = document;
    $(el).trigger(type, this);
    if (typeof value !== 'undefined' && value === 0) {
      this.rejectEvent(value);
    }
  };

  Drupal.Locker.prototype.hideLocker = function () {
    var iconLock = $(this.iconClass, this.holder);
    if (iconLock && iconLock.length) {
      iconLock.hide();
    }
    var lockerError = $(this.errorClass, this.holder);
    if (lockerError && lockerError.length) {
      lockerError.hide();
    }

    var lockerContent = $(this.contentClass, this.holder);
    if (lockerContent && lockerContent.length) {
      lockerContent.hide();
    }
    var wrapper = $('.cl-wrapper', this.holder);
    if (wrapper && wrapper.length) {
      wrapper.hide();
    }
  };

  Drupal.Locker.prototype.showLocker = function() {
    var wrapper = $('.cl-wrapper', this.holder);
    if (wrapper && wrapper.length) {
      wrapper.show();
    }
    else {
      throw new Error('Can not find wrapper.')
    }
  };
  
  Drupal.Locker.prototype.onUpdateNode = function (event, etype, nid) {
    if (!this.isCookie) {
      return false;
    }
    var options = null;
    if (this.cookieLife) {
      options = {
        expires: parseInt(this.cookieLife),
        path: '/'
      }
    }

    //save entity id to cookie for later;
    var old = {}, payload, result = {};
    var cdata = Drupal.cookies('lckr_' + this.type);
    if (cdata && cdata.length) {
      old = JSON.parse(cdata);
      old[etype][nid] = nid;
      payload = JSON.stringify(old);
      Drupal.cookies('lckr_' + this.type , payload, options);
    }
    else {
      result = {};
      result[etype] = {};
      result[etype][nid] = nid;
      payload = JSON.stringify(result);
      Drupal.cookies('lckr_' + this.type, payload, options);
    }

  };

  Drupal.Locker.prototype.onError = function (e, data) {
    var lockerType = data.type;
    if (lockerType === this.type) {
      var err = $('.' + this.errorClass, this.holder);
      if (err && err.length) {
        err.html(this.options.general.error);
        err.removeClass('element-hidden')
      }
    }
  };

  Drupal.Locker.prototype.hideError = function (e) {
    var err = $('.' + this.errorClass, $(this.holder));
    if (!err.hasClass('element-hidden')) {
      err.addClass('element-hidden');
    }
  }

  Drupal.Locker.prototype.subscribeEvents = function (el, obj) {
    var locker = obj;
    $(document).on(this.lockevent, function (e, source) {
      if (locker.type === source.type) {
        locker.lockEventAction(e, locker);
      }
    });
    $(document).on(this.unlockevent, function (e, source) {
      if (locker.type === source.type) {
        locker.unlockEventAction(e, locker, locker.type);
      }
    });
    $(document).on('contentLockerUpdateEntity', function(e, data){
      var uid = Drupal.userId();
      locker.onUpdateNode(e, data.etype, data.nid);
      locker.hideLocker();
    });

    $(document).on('contentLockerError', function (e, data) {
      locker.onError(e, data);
    })
  };

  Drupal.Locker.prototype.isProcessed = function (el) {
    var className = 'content-locker-processed';
    return !!(el.hasClass(className));
  };

  Drupal.Locker.prototype.getErrorMessage = function () {
    return this.errorMessage;
  };
  
  Drupal.getEntityData = function () {
    var lockers = $('.content-locker');
    if (lockers && lockers.length) {
      return {
        type: $(lockers[0]).data('content-locker-entitytype'),
        id: $(lockers[0]).data('content-locker-entityid')
      }
    }
    else {
      return false;
    }
  };

  Drupal.userId = function () {
    var uuid  = Drupal.cookies('user_twid');
    if (!uuid) {
      uuid = Drupal.guid();
      Drupal.cookies('user_twid', uuid);
    }
    return uuid;
  };

  /**
   * Generate unique user id.
   * @returns {string}
   */
  Drupal.guid = function () {
    function s4() {
      return Math.floor((1 + Math.random()) * 0x10000)
        .toString(16)
        .substring(1);
    }
    return s4() + s4() + '-' + s4() + '-' + s4() + '-' +
      s4() + '-' + s4() + s4() + s4();
  };

  Drupal.getLockerOptions = function (settings, locker, type) {
    switch (type) {
      case 'general':
        return settings.content_locker.plugins[locker].general;
      case 'base':
        return settings.content_locker.base;
      default:
        return settings.content_locker.plugins[locker];
    }
  };

  Drupal.exists = function (arr, obj, payload) {
    var found = false;
    arr.forEach(function(item, i) {
      if (item[payload] === obj[payload]
            && item.etype === obj.etype
              && obj.uuid === item.uuid) {
        found = true;
      }
    });
    return found;
  }

  Drupal.isTouch = function () {
    return !!('ontouchstart' in window)
      || !!('onmsgesturechange' in window);
  };

  Drupal.camelCase = function(input) {
    return input.toLowerCase().replace(/-(.)/g, function(match) {
      return match.toUpperCase();
    });
  };

  Drupal.capitaliseFirstLetter = function(input) {
    return input.charAt(0).toUpperCase() + input.slice(1);
  };

  Drupal.isMobile = function() {
    if ((/webOS|iPhone|iPod|BlackBerry/i).test(navigator.userAgent)) return true;
    if ((/Android/i).test(navigator.userAgent) && (/Mobile/i).test(navigator.userAgent)) return true;
    return false;
  };

  Drupal.isValidEmailAddress = function (emailAddress) {
    var pattern = new RegExp(/^((([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+(\.([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+)*)|((\x22)((((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(([\x01-\x08\x0b\x0c\x0e-\x1f\x7f]|\x21|[\x23-\x5b]|[\x5d-\x7e]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(\\([\x01-\x09\x0b\x0c\x0d-\x7f]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))))*(((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(\x22)))@((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.?$/i);
    return pattern.test(emailAddress);
  };

  Drupal.isValidUrl = function (url) {
    var pattern = new RegExp(/^(https?|ftp):\/\/(((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:)*@)?(((\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5]))|((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.?)(:\d*)?)(\/((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)+(\/(([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)*)*)?)?(\?((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|[\uE000-\uF8FF]|\/|\?)*)?(\#((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|\/|\?)*)?$/i);
    return pattern.test(url);
  };
  
  Drupal.cookies = function (key, value, options) {
    var defaultOptions = {
        expires: 365,
        path: '/'
    };
    if (!options || options.length === 0) {
      options = defaultOptions;
    }
    // Save cookie
    if (arguments.length > 1 && (!/Object/.test(Object.prototype.toString.call(value)) || value === null || value === undefined)) {
      options = $.extend({}, options);
      if (value === null || value === undefined) {
        options.expires = -1;
      }
      if (typeof options.expires !== 'number') {
        options.expires = parseInt(options.expires);
      }

      var days = options.expires, t = options.expires = new Date();
      t.setDate(t.getDate() + days);

      value = String(value);

      return (document.cookie = [
        encodeURIComponent(key), '=', options.raw ? value : encodeURIComponent(value),
        options.expires ? '; expires=' + options.expires.toUTCString() : '',
        options.path ? '; path=' + options.path : '',
        options.domain ? '; domain=' + options.domain : '',
        options.secure ? '; secure' : ''
      ].join(''));
    }

    // Receive cookie.
    options = value || {};
    var decode = options.raw ? function (s) { return s; } : decodeURIComponent;

    var pairs = document.cookie.split('; ');
    for (var i = 0, pair; pair = pairs[i] && pairs[i].split('='); i++) {
      if (decode(pair[0]) === key) return decode(pair[1] || '');
    }
    return null;
  };

  Drupal.saveValue = function( name, value, expires ) {
    if ( localStorage && localStorage.setItem ) {
      try {
        localStorage.setItem( name, value );
      } catch(e) {
        Drupal.cookie( name, value, { expires: expires, path: "/" });
      }
    } else {
      Drupal.cookie( name, value, { expires: expires, path: "/" });
    }
  };

  Drupal.getValue = function( name, defaultValue ) {
    var result = localStorage && localStorage.getItem && localStorage.getItem( name );
    if ( !result ) result = Drupal.cookie( name );
    if ( !result ) return defaultValue;
    return result;
  };

  Drupal.ucFirst = function (str) {
    return str.charAt(0).toUpperCase() + str.substr(1, str.length-1)
  }
  
  Drupal.elementIsHidden = function (el) {
    return (el.offsetParent === null) ? true : (window.getComputedStyle(el[0]).display === 'none')
  }

  Drupal.saveVisitorId = function(value) {
    Drupal.cookie("user_twid", value, {
      expires: 1000,
      path: "/"
    })
  };
  

  Drupal.updateQueryStr = function(uri, key, value) {
    var re = new RegExp("([?&])" + key + "=.*?(&|$)", "i");
    var separator = uri.indexOf('?') !== -1 ? "&" : "?";

    if (uri.match(re)) { return uri.replace(re, '$1' + key + "=" + value + '$2'); }
    else { return uri + separator + key + "=" + value; }
  };

  Drupal.prepareItem = function(etype, uid, nid) {
    var item = {};
    item[etype] = {}
    item[etype][nid] = nid;
    return item;
  }
})(jQuery, window, Drupal);


